<?php

namespace Drupal\gcal_entity;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use const _PHPStan_7d8742a37\__;

/**
 * Service for managing Google Calendar Entity settings.
 */
class GcalSettingsService {
  use StringTranslationTrait;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The settings config object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected StateInterface $state;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The configuration schema.
   *
   * @var array
   */
  protected $configSchema = [
    'googleapi' => ['type' => 'string', 'required' => TRUE],
    'start' => ['type' => 'string', 'required' => TRUE],
    'end' => ['type' => 'string', 'required' => TRUE],
    'datelimit' => ['type' => 'integer', 'required' => TRUE],
    'maxevents' => ['type' => 'integer', 'required' => TRUE],
    'dateformat' => ['type' => 'string', 'required' => TRUE],
    'timeformat' => ['type' => 'string', 'required' => TRUE],
    'noeventstext' => ['type' => 'string', 'required' => FALSE],
    'linktext' => ['type' => 'string', 'required' => FALSE],
    'hangoutlinktext' => ['type' => 'string', 'required' => FALSE],
    'cachetime' => ['type' => 'integer', 'required' => TRUE],
    'timezone' => ['type' => 'string', 'required' => TRUE],
    'api_storage_key' => ['type' => 'string', 'required' => FALSE],
  ];

  /**
   * Constructs a GcalSettingsService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    StateInterface $state,
    MessengerInterface $messenger
  ) {
    $this->configFactory = $config_factory;
    $this->config = $config_factory->get('gcal_entity.settings');
    $this->state = $state;
    $this->messenger = $messenger;
  }

  /**
   * Gets a configuration value.
   *
   * @param string $key
   *   The configuration key.
   * @param mixed $default
   *   (optional) The default value to return if the configuration value does not exist.
   *
   * @return mixed
   *   The configuration value, or the default value if the key doesn't exist.
   */
  public function get($key, $default = NULL) {
    return $this->config->get($key) ?? $default;
  }

  /**
   * Gets all configuration values.
   *
   * @return array
   *   All configuration values.
   */
  public function getAll() {
    return $this->config->get();
  }

  /**
   * Gets the configuration schema.
   *
   * @return array
   *   The configuration schema.
   */
  public function getSchema() {
    return $this->configSchema;
  }

  /**
   * Sets a configuration value.
   *
   * @param string $key
   *   The configuration key.
   * @param mixed $value
   *   The configuration value.
   *
   * @return bool
   *   TRUE if the value was set successfully, FALSE otherwise.
   */
  public function set($key, $value) {
    if (!$this->validateSetting($key, $value)) {
      return FALSE;
    }

    $config = $this->configFactory->getEditable('gcal_entity.settings');
    $config->set($key, $this->castValue($key, $value));
    $config->save();
    return TRUE;
  }

  /**
   * Sets multiple configuration values.
   *
   * @param array $values
   *   An associative array of configuration keys and values.
   *
   * @return array
   *   An array of keys that failed validation.
   */
  public function setMultiple(array $values): TranslatableMarkup {
    $failed_keys = [];
    $config = $this->configFactory->getEditable('gcal_entity.settings');
    $unchanged_keys = [];

    foreach ($values as $key => $value) {
      if (!$this->validateSetting($key, $value)) {
        $failed_keys[] = $key;
        continue;
      }
      // Check if the value in the config is different from the new value.
      $current_value = $this->castValue($key, $config->get($key));
      $casted_value = $this->castValue($key, $value);
      if ($current_value === $casted_value) {
        $unchanged_keys[] = $key;
        continue; // No need to set the same value.
      }
      $config->set($key, $this->castValue($key, $value));
    }
    $successful_keys = array_diff(array_keys($values), $failed_keys, $unchanged_keys);

    if (empty($failed_keys)) {
      $config->save();
      if(!empty($successful_keys)){
        return $this->t('The following items were successfully changed: @items' , ['@items' => implode(', ', $successful_keys)]);
      }
      else {
        return $this->t('No changes were made to interface values.');
      }
    }

    return $this->t('The following configuration keys had errors. Please fix and try saving again: @keys', [
      '@keys' => implode(', ', $failed_keys),
    ]);
  }

  /**
   * Validates a configuration setting.
   *
   * @param string $key
   *   The configuration key.
   * @param mixed $value
   *   The configuration value.
   *
   * @return bool
   *   TRUE if the setting is valid, FALSE otherwise.
   */
  public function validateSetting($key, $value) {
    // Check if the key exists in the schema
    if (!isset($this->configSchema[$key])) {
      $this->messenger->addError($this->t('Unknown configuration key: @key', [
        '@key' => $key,
      ]));
      return FALSE;
    }

    // Check if the setting is required and the value is empty
    if ($this->configSchema[$key]['required'] && (is_null($value) || $value === '')) {
      $this->messenger->addError($this->t('Configuration key @key is required but was empty.', [
        '@key' => $key,
      ]));
      return FALSE;
    }

    // Validate data type
    $type = $this->configSchema[$key]['type'];
    switch ($type) {
      case 'integer':
        if (!is_numeric($value) && !is_int($value)) {
          $this->messenger->addError($this->t('Configuration key @key must be a number.', [
            '@key' => $key,
          ]));
          return FALSE;
        }

        // For specific integer settings, perform additional validation
        if ($key === 'datelimit' || $key === 'maxevents') {
          if ((int) $value <= 0) {
            $this->messenger->addError($this->t('Configuration key @key must be a positive number.', [
              '@key' => $key,
            ]));
            return FALSE;
          }
        }
        if ($key === 'cachetime') {
          if ((int) $value < 0) {
            $this->messenger->addError($this->t('Configuration key @key cannot be negative.', [
              '@key' => $key,
            ]));
            return FALSE;
          }
        }
        break;

      case 'string':
        // Specific validation for time-related strings
        if ($key === 'start' || $key === 'end') {
          $timestamp = strtotime($value);
          if ($timestamp === FALSE) {
            $this->messenger->addError($this->t('Configuration key @key must be a valid date/time string.', [
              '@key' => $key,
            ]));
            return FALSE;
          }
        }

        // Validate timezone
        if ($key === 'timezone' && !in_array($value, timezone_identifiers_list())) {
          $this->messenger->addError($this->t('Configuration key @key must be a valid timezone identifier.', [
            '@key' => $key,
          ]));
          return FALSE;
        }
        break;
    }

    return TRUE;
  }

  /**
   * Casts a value to the correct type based on the schema.
   *
   * @param string $key
   *   The configuration key.
   * @param mixed $value
   *   The configuration value.
   *
   * @return mixed
   *   The cast value.
   */
  protected function castValue($key, $value) {
    if (!isset($this->configSchema[$key])) {
      return $value;
    }

    $type = $this->configSchema[$key]['type'];
    switch ($type) {
      case 'integer':
        return (int) $value;

      case 'string':
        return (string) $value;

      default:
        return $value;
    }
  }

  /**
   * Processes form values and updates configuration.
   *
   * @param array $values
   *   An associative array of configuration keys and values.
   *
   * @return bool
   *   TRUE if all values were successfully set, FALSE otherwise.
   */
  public function processFormValues(array $values): TranslatableMarkup {
    $results = $this->setMultiple($values);

  }

  /**
   * Gets the Google API key.
   *
   * @return string
   *   The Google API key.
   */
  public function getGoogleApiKey(): string {
    $storage_type = $this->getApiStorageType();
    if($storage_type === 'config'){
      $config = $this->configFactory->get('gcal_entity.settings');
      return $config->get('googleapi');
    } else {
      $api_key = $this->state->get('googleapi');
      if ($api_key) {
        return $api_key;
      }
      // If the state is not set, fall back to config.
      $config = $this->configFactory->get('gcal_entity.settings');
      return $config->get('googleapi');
    }
  }

  public function setApiKeyStorageType(string $storage_type) {
    // Storage type should be state or config.
    if(!in_array($storage_type, ['state', 'config'])) {
      $this->messenger->addError($this->t('Invalid storage type. Must be "state" or "config".'));
      return;
    }
    $this->set('api_storage_key', $storage_type);
  }

  public function getApiStorageType(): string {
    $default_type = 'config';
    return $this->get('api_storage_key', $default_type);
  }

  /**
   * Gets the event start time string.
   *
   * @return string
   *   The event start time string.
   */
  public function getEventStart() {
    return $this->get('start');
  }

  /**
   * Gets the event end time string.
   *
   * @return string
   *   The event end time string.
   */
  public function getEventEnd() {
    return $this->get('end');
  }

  /**
   * Gets the maximum number of dates to display.
   *
   * @return int
   *   The date limit.
   */
  public function getDateLimit() {
    return (int) $this->get('datelimit');
  }

  /**
   * Gets the maximum number of events to fetch.
   *
   * @return int
   *   The maximum events.
   */
  public function getMaxEvents() {
    return (int) $this->get('maxevents');
  }

  /**
   * Gets the date format.
   *
   * @return string
   *   The date format.
   */
  public function getDateFormat() {
    return $this->get('dateformat');
  }

  /**
   * Gets the time format.
   *
   * @return string
   *   The time format.
   */
  public function getTimeFormat() {
    return $this->get('timeformat');
  }

  /**
   * Gets the text to display when no events are found.
   *
   * @return string
   *   The no events text.
   */
  public function getNoEventsText() {
    return $this->get('noeventstext');
  }

  /**
   * Gets the calendar link text.
   *
   * @return string
   *   The calendar link text.
   */
  public function getLinkText() {
    return $this->get('linktext');
  }

  /**
   * Gets the hangout link text.
   *
   * @return string
   *   The hangout link text.
   */
  public function getHangoutLinkText() {
    return $this->get('hangoutlinktext');
  }

  /**
   * Gets the cache time in seconds.
   *
   * @return int
   *   The cache time in seconds.
   */
  public function getCacheTime() {
    return (int) $this->get('cachetime');
  }

  /**
   * Gets the timezone.
   *
   * @return string
   *   The timezone.
   */
  public function getTimezone() {
    return $this->get('timezone');
  }

  public function setGoogleApiKey(string $api_key): TranslatableMarkup {
    $default_message = $this->t('There was no change in the API key.');

    $storage_type = $this->getApiStorageType();
    if ($storage_type === 'config') {
      $config = $this->configFactory->getEditable('gcal_entity.settings');
      // ONly store if it has changed:
      $current_key = $config->get('googleapi');
      if ($current_key === $api_key) {
        return $default_message;
      }
        $config->set('googleapi', $api_key);
        $config->save();
    } else {
      // Store the API key in state.
      $this->state->set('googleapi', $api_key);
      // Only store if the value has changed.
      $current_key = $this->state->get('googleapi');
      if ($current_key !== $api_key) {
        return $default_message;

      }
      // Also unset the API key in config to avoid stale data.
      $config = $this->configFactory->getEditable('gcal_entity.settings');
      $config->clear('googleapi');
      $config->save();
    }
    return $this->t('API key was saved successfully to the @storage.', ['@storage' => $storage_type === 'config' ? 'configuration' : 'database']);

  }

}
